@@ -4,25 +4,22 @@ from troveclient import TroveAPI
 
 # You need a consumer.  Make one in the db. Consumers have call back URLs, which are not overridable (a-la Twitter and Fb)  
 c = { 
-	key: 'some_key_here',
-	secret: 'sssh, it\'s a secret!'
+	client_id: 'some_key_here',
+	secret: 'sssh, it\'s a secret!',
+	redirect_uri: 'http://redirecturl.dev/callback'
 }
 # Initialize the TroveAPI with the consumer's key and secret
-api = TroveAPI(c.key, c.secret, ['photo'])
+api = TroveAPI(client_id, secret, redirect_uri)
 
-rt = api.get_request_token()  
-# We get the request token -- returns back an 'OAuthToken'
- 
-url = api.get_authorization_url(rt)  
+url = api.get_authenticate_url() 
 print url
 raw_input()
 
-# Returns back the URL to redirect the user  (Looks like 'http://brooklyn.vlku.com:8000/multi/login?next=/oauth/authorize/%3Foauth_token%3DdtKaXn5JXRWNndTa')
+# Returns back the URL to redirect the user  (Looks like 'http://brooklyn.vlku.com:8000/multi/login?next=/oauth/2authorize/%3Foauth_token%3DdtKaXn5JXRWNndTa')
 
-# callback URL  AUTHORIZES (SAME) token and redirects to a page for you to act on token to get access token
-# to promote a request token to an access token call this method
+# callback URL includes a "code" that is passed in via the code parameter.  We use that to get an access_token.
 
-at = api.get_access_token(rt)  
+at = api.get_access_token(code)  
 # This returns back the access_token (at) as an OAuthToken.  You want to save this somewhere for reuse later
 # access_token also conveniently makes this the access_token for the instantiated API
 
