@@ -1,16 +1,20 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-"""    A Trove Client API interface using OAuth
+"""    
+    A Trove Client API interface using OAuth2
+    This is a big re-write using OAuth2 and /v2 API calls
 """
 __all__ = ('TroveAPI', 'TroveError',
         'get_request_token', 'get_authorization_url', 'get_access_token','get_photos', 'push_photos', '__version__')
 __author__ = 'Nick Vlku, Jesse Emery'
 __status__ = "Beta"
 __dependencies__ = ('python-dateutil', 'simplejson', 'urllib', 'urllib2', 'oauth')
-__version__ = '0.2.0'
+__version__ = '0.3.0'
 
-# This code is lovingly crafted in Brooklyn, NY (40°42′51″N, 73°57′12″W)
+# This code is lovingly crafted in 
+#   Brooklyn, NY (40°42′51″N, 73°57′12″W)
+#   and San Francisco, CA (37°46′38″ N, 122°24′40″ W)
 #
 # Typical MIT license below:
 #
@@ -41,7 +45,6 @@ import time
 import datetime
 import simplejson
 
-import oauth.oauth as oauth
 
 from urllib2 import HTTPError
 from dateutil.parser import *
@@ -49,31 +52,22 @@ from dateutil.parser import *
 from troveclient import JSONFactories
 from troveclient.JSONFactories import make_nice
 
-API_WEB_BASE = 'https://api.yourtrove.com'
 WEB_BASE = 'https://www.yourtrove.com'
+API_WEB_BASE = 'https://api.yourtrove.com'
 
-VERSION_WEB_BASE = '/v1'
+VERSION_WEB_BASE = '/v2'
+PREVIOUS_VERSION_WEB_BASE = "/v1"
 
-REQUEST_TOKEN_URL = WEB_BASE + VERSION_WEB_BASE + '/oauth/request_token/' 
-ACCESS_TOKEN_URL = WEB_BASE + VERSION_WEB_BASE + '/oauth/access_token/' 
-AUTHORIZATION_URL = WEB_BASE + VERSION_WEB_BASE + '/oauth/authorize/'
-CONTENT_ROOT_URL = API_WEB_BASE + VERSION_WEB_BASE +'/oauth/'
-PUSH_URL = API_WEB_BASE + VERSION_WEB_BASE + '/oauth/push/'
-USER_INFO_URL = API_WEB_BASE + VERSION_WEB_BASE +'/oauth/user/'
-ADD_URLS_FOR_SERVICES_URL = API_WEB_BASE + VERSION_WEB_BASE + '/oauth/get_add_urls_for_services/'
-CREATE_AND_AUTH_URL = API_WEB_BASE + VERSION_WEB_BASE + '/add/newtrove/service/'
+AUTHENTICATE_URL = WEB_BASE + '/oauth2/authenticate/'
+ACCESS_TOKEN_URL = WEB_BASE + '/oauth2/access_token/' 
 
-def _generate_nonce(length=8):
-    """Generate pseudorandom number."""
-    return ''.join([str(random.randint(0, 9)) for i in range(length)])
-
-def _generate_verifier(length=8):
-    """Generate pseudorandom number."""
-    return ''.join([str(random.randint(0, 9)) for i in range(length)])
+CONTENT_ROOT_URL = API_WEB_BASE + VERSION_WEB_BASE + '/content'
+USER_INFO_URL = API_WEB_BASE + VERSION_WEB_BASE +'/user/'
+ADD_URLS_FOR_SERVICES_URL = API_WEB_BASE + VERSION_WEB_BASE + '/services/'
+CREATE_AND_AUTH_URL = API_WEB_BASE + VERSION_WEB_BASE + '/services/%s/bounceback/'
 
 def _oauth_version():
-    """Right now always returns 1.0"""
-    return '1.0'
+    return '2'
 
 class TroveError():
     """The :class:`~troveclient.TroveError` is returned when the client receives back an error. It contains a standard http_error object
@@ -107,41 +101,30 @@ class LocalError():
 class TroveAPI():
     """The base class for all Trove client actions
     
-    :param consumer_key: A string of your application's Trove key
-    :param consumer_secret:  A string of your application's Trove secret
-    :param scope: A list of strings for scopes of the client.  Currently only 'photos' is valid.
-    :param access_token: An instance of :class:`~oauth.oauth.OAuthToken' that is used to make requests on behalf of a user
-    :param authorization_callback: URL that the user should be returned to when they are asked to authorize (can be overridden be specifying in get_authorization_url)
+    :param client_id: A string of your application's YourTrove client ID
+    :param client_secret:  A string of your application's YourTrove client Secret
+    :param redirect_uri: URL that the user should be returned to when they are asked to authorize.  Needs to match your application settings
+    :param scope: A list of strings for scopes of the client.Valid options: 'photos', 'checkins', 'status'
+    :param access_token: An access token string that is used to make requests on behalf of a user
     """
 
     DEBUG = False
 
-    def __init__(self, consumer_key, consumer_secret,scope=None, access_token=None, authorization_callback=None):
-        self._Consumer = oauth.OAuthConsumer(consumer_key, consumer_secret)
-        self._signature_method = oauth.OAuthSignatureMethod_HMAC_SHA1
+    def __init__(self, client_id, client_secret, redirect_uri, scope=None, access_token=None):
         self._access_token = access_token
         if scope is not None:
             self._scope = scope
         else:
-            # set the default to photos, let's be real
-            self._scope = ['photos']
+            # se the default to all 
+            self._scope = ['photos', 'checkins', 'status']
+        self.client_id = client_id
+        self.client_secret = client_secret
         self._urllib = urllib2
-        self.authorization_callback = authorization_callback
+        self.redirect_uri = redirect_uri
         self._initialize_user_agent()
-        self._default_oauth_params = {
-                  'oauth_consumer_key': self._Consumer.key,
-                  'oauth_signature_method': 'PLAINTEXT',
-                  'oauth_nonce':  _generate_nonce(),
-                  'oauth_version': _oauth_version(),
-                  'scope': ','.join(self._scope)
-        } 
-        if access_token is None:
-            self.get_request_token()
-            self.get_authorization_url(self.request_token, self.authorization_callback)
-        
         
     def _initialize_user_agent(self):
-        user_agent = 'Python-urllib/%s (python-trove/%s)' % \
+        user_agent = 'YourTrove Python Client - Python-urllib/%s (python-trove/%s)' % \
                      (self._urllib.__version__, __version__)
         self.set_user_agent(user_agent)
                 
@@ -150,31 +133,10 @@ class TroveAPI():
         
     def set_user_agent(self, user_agent):
         self._useragent = user_agent
-    
-    def __get_default_oauth_params(self):
-        parameters = self._default_oauth_params.copy()
-        parameters['oauth_nonce'] = _generate_nonce()
-        parameters['oauth_timestamp'] = str(int(time.time()))
-
-        return parameters
-    
-    def __make_oauth_request(self, url, parameters=None, oauth_signature=None, token=None, signed=False, method="POST"):
-        if parameters is None:
-            parameters = self.__get_default_oauth_params()
+        
+    def __make_oauth_request(self, url, parameters=None, token=None, method="POST"):
         if token is not None:
-            parameters['oauth_token'] = token.key
-        if signed:
-            parameters['oauth_signature_method'] = 'HMAC-SHA1'
-            oauthrequest = oauth.OAuthRequest.from_token_and_callback(self._access_token, http_url=url, parameters=parameters, http_method=method)
-            signature_method = self._signature_method()
-            signature = signature_method.build_signature(oauthrequest, self._Consumer, self._access_token)
-            if self.DEBUG:
-                print 'OAuth Signature:'
-                print signature
-            parameters['oauth_signature'] = signature
-        else:
-            if oauth_signature is not None:
-                parameters['oauth_signature'] = oauth_signature
+            parameters['access_token'] = access_token
 
         try:
             if method is "POST":
@@ -199,46 +161,68 @@ class TroveAPI():
         except HTTPError, e:
             error = TroveError(e, request)
             raise error
-    
-    def get_request_token(self):
-        """    Returns an instance of :class:`~oauth.oauth.OAuthToken` representing a Request Token for user. 
-        """    
-        response = self.__make_oauth_request(REQUEST_TOKEN_URL, oauth_signature=self._Consumer.secret+"&", method="GET")
-        self.response = response
-        request_token = oauth.OAuthToken.from_string(response.read())
-        self.request_token = request_token
-        return request_token
 
-    def get_authorization_url(self, request_token=None, callback=None):
-        """      Returns an authorization URL that should be passed to the end user to authorize their request token.
-                
-                :param request_token: An instance of :class:`~oauth.oauth.OAuthToken` that is used to authorize an access token by the user
-                :param callback: A URL for the service to bounceback to.  The default is the application's callback URL specified in Trove
+    def get_authenticate_url(self):
+        """      Returns an authentication URL that the user should be redirected to, so they can authorize your app.
+            
         """
-        if request_token is None and self.request_token is not None:
-            request_token = self.request_token
                     
-        parameters = { 'oauth_token': request_token.key }
+        parameters = {  'client_id'     : self.client_id,
+                        'response_type' : 'code',
+                        'redirect_uri'  : self.redirect_uri 
+                        }
+        encoded_params = urllib.urlencode(parameters)
+        return str.join("",[AUTHENTICATE_URL, "?", encoded_params])
+    
+    def __make_oauth2_request(self, base_url, parameters=None, signed=False, post=False):
+        try:
+            if signed and self._access_token is None:
+                raise RequiresAccessTokenError()
+
+            if parameters is not None:
+                copied_params = parameters.copy()
+            else:
+                copied_params = {}
         
-        if callback:
-            parameters['oauth_callback'] = callback
-        elif self.authorization_callback:
-            parameters['oauth_callback'] = self.authorization_callback
-
-        response = self.__make_oauth_request(AUTHORIZATION_URL, parameters, oauth_signature=None, method="GET")
-        self.authorization_url = response.geturl()            
-        return self.authorization_url
+            if signed:
+                copied_params['access_token'] = self._access_token
+    
+            encoded_params = urllib.urlencode(copied_params)
+        
+            if post:
+                request = self._urllib.Request(base_url)
+                request.add_header('User-agent', self._useragent)
+                if self.DEBUG:
+                    print base_url + " : Parameters=" + copied_params.__str__() + " : Encoded Parameters=" + encoded_params
+                    raw_input()
+                return self._urllib.urlopen(request, encoded_params)
+            else:
+                request = self._urllib.Request(base_url + '?' + encoded_params)  
+                request.add_header('User-agent', self._useragent)
+                if self.DEBUG:
+                    print base_url + '?' + encoded_params
+                    raw_input()
                 
-    def get_access_token(self, request_token=None):
-        """      Returns an instance of :class:`~oauth.oauth.OAuthToken` that is used to make requests on behalf of a user
+                return self._urllib.urlopen(request)
+        except HTTPError, e:
+            error = TroveError(e, request)
+            raise error
 
-                :param request_token: An instance of :class:`~oauth.oauth.OAuthToken` that is used to authorize an access token by the user
+    def get_access_token(self, code=None):
+        """      Returns an access_token string that is used to make requests on behalf of a user
+
+                :param code: A string representing the code returned from a user auth request
         """
-        if request_token is None and self.request_token is not None:
-            request_token = self.request_token
-        response = self.__make_oauth_request(ACCESS_TOKEN_URL, oauth_signature='%s&%s' % (self._Consumer.secret, request_token.secret), token=request_token, method='GET')
-        self.response = response
-        access_token = oauth.OAuthToken.from_string(response.read())
+        parameters = {  'client_id'     : self.client_id,
+                        'grant_type' : 'authorization_code',
+                        'redirect_uri'  : self.redirect_uri,
+                        'client_secret' : self.client_secret,
+                        'code'          : code
+                        }
+        response = self.__make_oauth2_request(ACCESS_TOKEN_URL, parameters=parameters)
+
+        r = simplejson.loads(response.read())
+        access_token = r.get('access_token')
         self._access_token = access_token
         return access_token
         
@@ -251,31 +235,50 @@ class TroveAPI():
 
         if self._access_token is None:
             raise RequiresAccessTokenError()
-        response = self.__make_oauth_request(USER_INFO_URL, token=self._access_token, signed=True)
+        response = self.__make_oauth2_request(USER_INFO_URL, signed=True)
+        
         return simplejson.loads(response.read())
+    
+    def __get_internal(self, query=None, content_type='photos'):
+        if self._access_token is None:
+            raise RequiresAccessTokenError()
+        
+        base_url = CONTENT_ROOT_URL + "/" + content_type + '/'
+        parameters = {}
+        if query is not None:
+            query_post = simplejson.dumps(query, cls=JSONFactories.encoders.get_encoder_for(query))
+            parameters['query'] = query_post
 
+        response = self.__make_oauth2_request(base_url, parameters=parameters, signed=True, post=True)
+        
+        results = simplejson.loads(response.read())
+        results = make_nice.make_it_nice(results)
+        return results
+        
     def get_photos(self,query=None): 
         """     Retrieves a user's photos based on the Query passed in. *This call requires an access token.*
                 Returns a :class:`~troveclient.Objects.Result` object that contains the results of your query
                 
                 :param query: An instance of :class:~`troveclient.Objects.Query~ object.  If none is passed, assumes page 1 with a count of 50.
         """
-        if self._access_token is None:
-            raise RequiresAccessTokenError()
+        return self.__get_internal(query)
         
-        parameters = self.__get_default_oauth_params()
-        base_url = CONTENT_ROOT_URL + 'photos/'
+    def get_checkins(self,query=None): 
+        """     Retrieves a user's checkins based on the Query passed in. *This call requires an access token.*
+                Returns a :class:`~troveclient.Objects.Result` object that contains the results of your query
+                
+                :param query: An instance of :class:~`troveclient.Objects.Query~ object.  If none is passed, assumes page 1 with a count of 50.
+        """
+        return self.__get_internal(query,'checkins')
+
+    def get_status(self,query=None): 
+        """     Retrieves a user's status updates based on the Query passed in. *This call requires an access token.*
+                Returns a :class:`~troveclient.Objects.Result` object that contains the results of your query
+                
+                :param query: An instance of :class:~`troveclient.Objects.Query~ object.  If none is passed, assumes page 1 with a count of 50.
+        """
+        return self.__get_internal(query,'status')
 
-        if query is not None:
-            query_post = simplejson.dumps(query, cls=JSONFactories.encoders.get_encoder_for(query))
-            parameters['query'] = query_post
-            self.response = self.__make_oauth_request(base_url, parameters, token=self._access_token, signed=True, method="POST")
-        else:
-            self.response = self.__make_oauth_request(base_url, parameters, token=self._access_token, signed=True, method="GET")
-        
-        results = simplejson.loads(self.response.read())
-        nice_result = make_nice.make_it_nice(results)
-        return nice_result
 
     def push_photos(self, user_id,  photos_list= []):
         """     Pushes a list of photos for user_id specified into Trove associated to your service. *This call requires an access token.*
@@ -290,15 +293,17 @@ class TroveAPI():
         if photos_list is None: 
             return
         
-        parameters = self.__get_default_oauth_params()
-
+        parameters = {}
         json_photos_list = simplejson.dumps(photos_list, cls=JSONFactories.encoders.get_encoder_for(photos_list[0]))
                                             
         parameters['object_list'] = json_photos_list
         parameters['number_of_items'] = len(photos_list)
         parameters['content_type'] = 'photos'
         parameters['user_id'] = user_id
-        response = self.__make_oauth_request(PUSH_URL, parameters, token=self._access_token, signed=True, method="POST")
+        
+        base_url = CONTENT_ROOT_URL + "/" + "photos" + '/'
+        
+        response = self.__make_oauth2_request(base_url, parameters=parameters, signed=True, post=True)
 
         return simplejson.loads(response.read())
         
@@ -309,10 +314,14 @@ class TroveAPI():
         if self._access_token is None:
             raise RequiresAccessTokenError()
 
-        response = self.__make_oauth_request(ADD_URLS_FOR_SERVICES_URL, token=self._access_token, signed=True)
-        return simplejson.loads(response.read()).keys()    
+        response = self.__make_oauth2_request(ADD_URLS_FOR_SERVICES_URL, signed=True)
+        services_json = simplejson.loads(response.read())
+        service_decoder = make_nice.get_decoder_for("service")
+        services = [service_decoder.get_object(service_json) for service_json in services_json]
+            
+        return services
     
-    def get_url_for_service(self, service, redirect_url=None):
+    def get_url_for_service(self, service_name, redirect_url=None):
         """     This creates a URL that allows a user to add a service to their Trove and then immediately bounce back to your service.  If a 
                 redirect_url URL is specified it uses that, otherwise it uses your application's default.  *This call requires an access token.*
                 Returns a one-time use URL for the user
@@ -320,31 +329,44 @@ class TroveAPI():
                 :param service: The service name, usually from the list returned by get_services
                 :param redirect_url: A redirect URL override.  The default is your application's callback.
         """        
-
-        if self._access_token is None:
-            raise RequiresAccessTokenError()
-
-        response = self.__make_oauth_request(ADD_URLS_FOR_SERVICES_URL, token=self._access_token, signed=True)
-        services = simplejson.loads(response.read())
-            
-        if service in services.keys():
-            service_url = API_WEB_BASE + services[service]
-            parameters = self.__get_default_oauth_params() 
+        
+        services = self.get_services()
+        found_it = False
+        for service in services:
+            if str.lower(str(service.name)) == str.lower(str(service_name)):
+                found_it = True
+                break
+        if found_it:            
+            parameters = {}
             if redirect_url is not None:
                 parameters['redirect_url'] = redirect_url
-
-            response = self.__make_oauth_request(service_url, parameters, token=self._access_token, signed=True)
-            return WEB_BASE + response.read()
+            else:
+                parameters['redirect_url'] = self.redirect_uri
+            url = API_WEB_BASE + VERSION_WEB_BASE + "/services/%s/bounceback/"  % (str.lower(service.name),)
+            response = self.__make_oauth2_request(url, parameters=parameters, signed=True)
+            return response.read()
         else: 
-            raise LocalError("Could not find service name " + service)
+            raise LocalError("Could not find service name " + service_name)
         
-    def login_and_auth_via_service(self, service):
+    def login_and_auth_via_service(self, service, redirect_url=None):
+        """     This creates a URL that allows a user to create a Trove account implicitly by adding this service to their Trove.
+                It immediately bounces back to your service.  If a  redirect_url URL is specified it uses that, otherwise it uses your
+                application's default.  *This call requires an access token.*
+                
+                Returns a one-time use URL for the user
+                
+                :param service: The service name, usually from the list returned by get_services
+                :param redirect_url: A redirect URL override.  The default is your application's callback.
+        """        
+
         parameters = {}
-        parameters['app_key'] = self._Consumer.key
-        parameters['app_secret'] = self._Consumer.secret
+        parameters['client_id'] = self.client_id
+        if redirect_url is None:
+            redirect_url = self.redirect_uri
+        parameters['redirect_url'] = redirect_url
         
         encoded_parameters = urllib.urlencode(parameters)
         
-        req = urllib2.Request(CREATE_AND_AUTH_URL + service, encoded_parameters)
+        req = urllib2.Request(API_WEB_BASE + VERSION_WEB_BASE + "/services/%s/login_and_auth/?%s"  % (service, encoded_parameters))
         self.response = urllib2.urlopen(req)
-        return WEB_BASE  + self.response.read()
+        return self.response.read()
